@@ -1,6 +1,6 @@
 /* This testcase is part of GDB, the GNU debugger.
 
-   Copyright 2010, 2011 Free Software Foundation, Inc.
+   Copyright 2010-2014 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,8 +13,9 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see  <http://www.gnu.org/licenses/>.
-*/
+   along with this program.  If not, see  <http://www.gnu.org/licenses/>.  */
+
+#include <unistd.h>
 
 extern void do_nothing (void);
 
@@ -24,6 +25,11 @@ int second(){
 }
 
 int first(){
+  int i;
+
+  for (i = 0; i < 2; i++)
+    do_nothing ();
+
   return second();
 }
 
